@@ -53,6 +53,7 @@
    and for each one call FUNCTION with the following arguments:
      archive file descriptor for reading the data,
      member name,
+     member name might be truncated flag,
      member header position in file,
      member data position in file,
      member data size,
@@ -82,6 +83,8 @@ ar_scan (archive, function, arg)
 #ifdef AIAMAG
   FL_HDR fl_header;
 #endif
+  char *namemap = 0;
+  int long_name = 0;
   register int desc = open (archive, O_RDONLY, 0);
   if (desc < 0)
     return -1;
@@ -153,7 +156,9 @@ ar_scan (archive, function, arg)
 	int uidval, gidval;
 	long int data_offset;
 #else
-	char name[sizeof member_header.ar_name + 1];
+	char namebuf[sizeof member_header.ar_name + 1];
+	char *name;
+	int is_namemap;		/* Nonzero if this entry maps long names.  */
 #endif
 	long int eltsize;
 	int eltmode;
@@ -186,6 +191,8 @@ ar_scan (archive, function, arg)
 	
 	name[name_len] = 0;
 
+	long_name = 1;
+
 	sscanf (member_header.ar_date, "%12ld", &dateval);
 	sscanf (member_header.ar_uid, "%12d", &uidval);
 	sscanf (member_header.ar_gid, "%12d", &gidval);
@@ -216,15 +223,59 @@ ar_scan (archive, function, arg)
 	    return -2;
 	  }
 
+	name = namebuf;
 	bcopy (member_header.ar_name, name, sizeof member_header.ar_name);
 	{
 	  register char *p = name + sizeof member_header.ar_name;
-	  while (p > name && *--p == ' ')
+	  do
 	    *p = '\0';
+	  while (p > name && *--p == ' ');
 
+#ifndef AIAMAG
+	  /* If the member name is "//" or "ARFILENAMES/" this may be
+	     a list of file name mappings.  The maximum file name
+ 	     length supported by the standard archive format is 14
+ 	     characters.  This member will actually always be the
+ 	     first or second entry in the archive, but we don't check
+ 	     that.  */
+ 	  is_namemap = (!strcmp (name, "//")
+			|| !strcmp (name, "ARFILENAMES/"));
+#endif	/* Not AIAMAG. */
 	  /* On some systems, there is a slash after each member name.  */
 	  if (*p == '/')
 	    *p = '\0';
+
+#ifndef AIAMAG
+ 	  /* If the member name starts with a space or a slash, this
+ 	     is an index into the file name mappings (used by GNU ar).
+ 	     Otherwise if the member name looks like #1/NUMBER the
+ 	     real member name appears in the element data (used by
+ 	     4.4BSD).  */
+ 	  if (! is_namemap
+ 	      && (name[0] == ' ' || name[0] == '/')
+ 	      && namemap != 0)
+	    {
+	      name = namemap + atoi (name + 1);
+	      long_name = 1;
+	    }
+ 	  else if (name[0] == '#'
+ 		   && name[1] == '1'
+ 		   && name[2] == '/')
+ 	    {
+ 	      int namesize = atoi (name + 3);
+ 
+ 	      name = (char *) alloca (namesize + 1);
+ 	      nread = read (desc, name, namesize);
+ 	      if (nread != namesize)
+ 		{
+ 		  close (desc);
+ 		  return -2;
+ 		}
+ 	      name[namesize] = '\0';
+
+	      long_name = 1;
+ 	    }
+#endif /* Not AIAMAG. */
 	}
 
 #ifndef	M_XENIX
@@ -236,7 +287,7 @@ ar_scan (archive, function, arg)
 #endif	/* Not Xenix.  */
 
 	fnval =
-	  (*function) (desc, name, member_offset,
+	  (*function) (desc, name, ! long_name, member_offset,
 		       member_offset + AR_HDR_SIZE, eltsize,
 #ifndef	M_XENIX
 		       atol (member_header.ar_date),
@@ -270,6 +321,40 @@ ar_scan (archive, function, arg)
 	    return -2;
 	  }
 #else
+
+ 	/* If this member maps archive names, we must read it in.  The
+ 	   name map will always precede any members whose names must
+ 	   be mapped.  */
+	if (is_namemap)
+ 	  {
+ 	    char *clear;
+ 	    char *limit;
+
+ 	    namemap = (char *) alloca (eltsize);
+ 	    nread = read (desc, namemap, eltsize);
+ 	    if (nread != eltsize)
+ 	      {
+ 		(void) close (desc);
+ 		return -2;
+ 	      }
+ 
+ 	    /* The names are separated by newlines.  Some formats have
+ 	       a trailing slash.  Null terminate the strings for
+ 	       convenience.  */
+ 	    limit = namemap + eltsize;
+ 	    for (clear = namemap; clear < limit; clear++)
+ 	      {
+ 		if (*clear == '\n')
+ 		  {
+ 		    *clear = '\0';
+ 		    if (clear[-1] == '/')
+ 		      clear[-1] = '\0';
+ 		  }
+ 	      }
+ 
+	    is_namemap = 0;
+ 	  }
+
 	member_offset += AR_HDR_SIZE + eltsize;
 	if (member_offset % 2 != 0)
 	  member_offset++;
@@ -281,12 +366,14 @@ ar_scan (archive, function, arg)
   return 0;
 }
 
-/* Return nonzero iff NAME matches MEM.  If NAME is longer than
-   sizeof (struct ar_hdr.ar_name), MEM may be the truncated version.  */
+/* Return nonzero iff NAME matches MEM.
+   If TRUNCATED is nonzero, MEM may be truncated to
+   sizeof (struct ar_hdr.ar_name) - 1.  */
 
 int
-ar_name_equal (name, mem)
+ar_name_equal (name, mem, truncated)
      char *name, *mem;
+     int truncated;
 {
   char *p;
 
@@ -294,7 +381,10 @@ ar_name_equal (name, mem)
   if (p != 0)
     name = p + 1;
 
-#if !defined (AIAMAG) && !defined (APOLLO)
+  /* We no longer use this kludge, since we
+     now support long archive member names.  */
+
+#if 0 && !defined (AIAMAG) && !defined (APOLLO)
 
   {
     /* `reallylongname.o' matches `reallylongnam.o'.
@@ -323,6 +413,18 @@ ar_name_equal (name, mem)
 
 #else	/* AIX or APOLLO.  */
 
+  if (truncated)
+    {
+#ifdef AIAMAG
+      /* TRUNCATED should never be set on this system.  */
+      abort ();
+#else
+      struct ar_hdr hdr;
+      return !strncmp (name, mem,
+		       sizeof (hdr.ar_name) - 1);
+#endif
+    }
+
   return !strcmp (name, mem);
 
 #endif
@@ -330,14 +432,16 @@ ar_name_equal (name, mem)
 
 /* ARGSUSED */
 static long int
-ar_member_pos (desc, mem, hdrpos, datapos, size, date, uid, gid, mode, name)
+ar_member_pos (desc, mem, truncated,
+	       hdrpos, datapos, size, date, uid, gid, mode, name)
      int desc;
      char *mem;
+     int truncated;
      long int hdrpos, datapos, size, date;
      int uid, gid, mode;
      char *name;
 {
-  if (!ar_name_equal (name, mem))
+  if (!ar_name_equal (name, mem, truncated))
     return 0;
   return hdrpos;
 }
@@ -408,15 +512,19 @@ ar_member_touch (arname, memname)
 #ifdef TEST
 
 long int
-describe_member (desc, name, hdrpos, datapos, size, date, uid, gid, mode)
+describe_member (desc, name, truncated,
+		 hdrpos, datapos, size, date, uid, gid, mode)
      int desc;
      char *name;
+     int truncated;
      long int hdrpos, datapos, size, date;
      int uid, gid, mode;
 {
   extern char *ctime ();
 
-  printf ("Member %s: %ld bytes at %ld (%ld).\n", name, size, hdrpos, datapos);
+  printf ("Member `%s'%s: %ld bytes at %ld (%ld).\n",
+	  name, truncated ? " (name might be truncated)" : "",
+	  size, hdrpos, datapos);
   printf ("  Date %s", ctime (&date));
   printf ("  uid = %d, gid = %d, mode = 0%o.\n", uid, gid, mode);
 
