@@ -1119,6 +1119,11 @@ handle_function (op, stringp)
 	    break;
 	}
 
+      if (count >= 0)
+	makefile_fatal (reading_filename, *reading_lineno_ptr,
+			"unterminated call to function `%s': missing `%c'",
+			function_table[code].name, closeparen);
+
       /* We found the end; expand the function call.  */
 
       *op = expand_function (*op, function_table[code].function, argbeg, p);
