@@ -39,6 +39,7 @@
    BSD				Real BSD, not just BSD-like.
    DGUX
    eunice			UNIX emulator under VMS.
+   hpux
    NeXT
    sgi
    sequent			Sequent Dynix 3.x.x (BSD)
@@ -103,6 +104,11 @@
 
 
 /* Some shorthands.  */
+
+#if defined (HPUX) && !defined (hpux)
+#define hpux
+#endif
+
 #if defined(hp300) && !defined(hpux)
 #define MORE_BSD
 #endif
@@ -119,8 +125,16 @@
 #define OSF_ALPHA
 #endif
 
+/* UTek's /bin/cc on the 4300 has no architecture specific cpp define by
+   default, but _MACH_IND_SYS_TYPES is defined in <sys/types.h>.  Combine
+   that with a couple of other things and we'll have a unique match.  */
+#if !defined (tek4300) && defined (unix) && defined (m68k) && defined (mc68000) && defined (mc68020) && defined (_MACH_IND_SYS_TYPES)
+#define tek4300			/* Define by emacs, but not by other users.  */
+#endif
+
+
 /* VAX C can't handle multi-line #ifs.  */
-#if (defined(MORE_BSD) || defined(sun) || defined(decstation) || defined(_SEQUENT_) || defined(sgi) || defined(SVR4) || defined(sony_news) || defined(sequent) || defined (OSF_ALPHA))
+#if (defined(MORE_BSD) || defined(sun) || defined(decstation) || defined(_SEQUENT_) || defined(sgi) || defined(SVR4) || defined(sony_news) || defined(sequent) || defined (OSF_ALPHA) || (defined (ardent) && defined (titan)) || defined (tek4300))
 #define LOAD_AVE_TYPE long
 #endif
 
@@ -141,6 +155,14 @@
 #define	FSCALE 1000.0
 #endif
 
+#if defined (ardent) && defined (titan)
+#define FSCALE 65536.0
+#endif
+
+#ifdef tek4300
+#define FSCALE 100.0
+#endif
+
 #endif	/* Not FSCALE.  */
 
 #if !defined (LDAV_CVT) && defined (FSCALE)
@@ -148,7 +170,7 @@
 #endif
 
 /* VAX C can't handle multi-line #ifs.  */
-#if !defined(NLIST_STRUCT) && (defined(MORE_BSD) || defined(sun) || defined(decstation) || defined(hpux) || defined(_SEQUENT_) || defined(sequent) || defined(sgi) || defined(SVR4) || defined(sony_news) || defined (OSF_ALPHA))
+#if !defined(NLIST_STRUCT) && (defined(MORE_BSD) || defined(sun) || defined(decstation) || defined(hpux) || defined(_SEQUENT_) || defined(sequent) || defined(sgi) || defined(SVR4) || defined(sony_news) || defined (OSF_ALPHA) || (defined (ardent) && defined (titan)) || defined (tek4300))
 #define NLIST_STRUCT
 #endif
 
@@ -166,7 +188,7 @@
 #define KERNEL_FILE "/hp-ux"
 #endif
 
-#if !defined(KERNEL_FILE) && (defined(_SEQUENT_) || defined(MIPS) || defined(SVR4) || defined(ISC) || defined (sgi) || defined(SVR4))
+#if !defined(KERNEL_FILE) && (defined(_SEQUENT_) || defined(MIPS) || defined(SVR4) || defined(ISC) || defined (sgi) || defined(SVR4) || (defined (ardent) && defined (titan)))
 #define KERNEL_FILE "/unix"
 #endif
 
@@ -175,7 +197,7 @@
 #define LDAV_SYMBOL "_Loadavg"
 #endif
 
-#if !defined(LDAV_SYMBOL) && (defined(hpux) || defined(_SEQUENT_) || defined(SVR4) || defined(ISC) || defined(sgi))
+#if !defined(LDAV_SYMBOL) && ((defined(hpux) && !defined(hp9000s300)) || defined(_SEQUENT_) || defined(SVR4) || defined(ISC) || defined(sgi) || (defined (ardent) && defined (titan)))
 #define LDAV_SYMBOL "avenrun"
 #endif
 
@@ -235,7 +257,11 @@ extern int errno;
 #endif /* LOAD_AVE_TYPE */
 
 #ifdef NeXT
+#ifdef HAVE_MACH_MACH_H
 #include <mach/mach.h>
+#else
+#include <mach.h>
+#endif
 #endif /* NeXT */
 
 #ifdef sgi
@@ -279,7 +305,7 @@ extern int errno;
 
 #ifdef NeXT
 static processor_set_t default_set;
-static int initialized;
+static int getloadavg_initialized;
 #endif /* NeXT */
 
 #ifdef UMAX
@@ -295,7 +321,7 @@ static struct dg_sys_info_load_info load_info;	/* what-a-mouthful! */
 /* File descriptor open to /dev/kmem or VMS load ave driver.  */
 static int channel;
 /* Nonzero iff channel is valid.  */
-static int initialized;
+static int getloadavg_initialized;
 /* Offset in kmem to seek to read load average, or 0 means invalid.  */
 static long offset;
 
@@ -336,19 +362,19 @@ getloadavg (loadavg, nelem)
       return -1;
     }
 
-  if (!initialized)
+  if (!getloadavg_initialized)
     {
       if (processor_set_default (host_self (), &default_set) == KERN_SUCCESS)
-	initialized = 1;
+	getloadavg_initialized = 1;
     }
 
-  if (initialized)
+  if (getloadavg_initialized)
     {
       info_count = PROCESSOR_SET_BASIC_INFO_COUNT;
       if (processor_set_info (default_set, PROCESSOR_SET_BASIC_INFO, &host,
 			     (processor_set_info_t) &info, &info_count)
 	  != KERN_SUCCESS)
-	initialized = 0;
+	getloadavg_initialized = 0;
       else
 	{
 	  if (nelem > 0)
@@ -356,7 +382,7 @@ getloadavg (loadavg, nelem)
 	}
     }
 
-  if (!initialized)
+  if (!getloadavg_initialized)
     return -1;
 #endif /* NeXT */
 
@@ -473,11 +499,10 @@ getloadavg (loadavg, nelem)
 #endif /* apollo */
 
 #if !defined (LDAV_DONE) && defined (VMS)
-#define LDAV_DONE
   /* VMS specific code -- read from the Load Ave driver.  */
 
   LOAD_AVE_TYPE load_ave[3];
-  static int initialized = 0;
+  static int getloadavg_initialized = 0;
 #ifdef eunice
   struct
   {
@@ -487,7 +512,7 @@ getloadavg (loadavg, nelem)
 #endif
 
   /* Ensure that there is a channel open to the load ave device.  */
-  if (!initialized)
+  if (!getloadavg_initialized)
     {
       /* Attempt to open the channel.  */
 #ifdef eunice
@@ -497,23 +522,24 @@ getloadavg (loadavg, nelem)
       $DESCRIPTOR (descriptor, "LAV0:");
 #endif
       if (sys$assign (&descriptor, &channel, 0, 0) & 1)
-	initialized = 1;
+	getloadavg_initialized = 1;
     }
 
   /* Read the load average vector.  */
-  if (initialized && !(sys$qiow (0, channel, IO$_READVBLK, 0, 0, 0,
-				 load_ave, 12, 0, 0, 0, 0) & 1))
+  if (getloadavg_initialized
+      && !(sys$qiow (0, channel, IO$_READVBLK, 0, 0, 0,
+		     load_ave, 12, 0, 0, 0, 0) & 1))
     {
       sys$dassgn (channel);
-      initialized = 0;
+      getloadavg_initialized = 0;
     }
 
-  if (!initialized)
+  if (!getloadavg_initialized)
     return -1;
 #endif /* VMS */
 
 #if !defined (LDAV_DONE) && defined(LOAD_AVE_TYPE) && !defined(VMS)
-#define LDAV_DONE
+
   /* UNIX-specific code -- read the average from /dev/kmem.  */
 
 #define LDAV_PRIVILEGED		/* This code requires special installation.  */
@@ -557,24 +583,24 @@ getloadavg (loadavg, nelem)
     }
 
   /* Make sure we have /dev/kmem open.  */
-  if (!initialized)
+  if (!getloadavg_initialized)
     {
 #ifndef SUNOS_5
       channel = open ("/dev/kmem", 0);
       if (channel >= 0)
-	initialized = 1;
+	getloadavg_initialized = 1;
 #else /* SUNOS_5 */
       kd = kvm_open (0, 0, 0, O_RDONLY, 0);
       if (kd != 0) 
 	{
 	  kvm_nlist (kd, nl);
-	  initialized = 1;
+	  getloadavg_initialized = 1;
 	}
 #endif /* SUNOS_5 */
     }
 
   /* If we can, get the load average values.  */
-  if (offset && initialized)
+  if (offset && getloadavg_initialized)
     {
       /* Try to read the load.  */
 #ifndef SUNOS_5
@@ -583,19 +609,19 @@ getloadavg (loadavg, nelem)
 	  != sizeof (load_ave))
 	{
 	  close (channel);
-	  initialized = 0;
+	  getloadavg_initialized = 0;
 	}
 #else  /* SUNOS_5 */
       if (kvm_read (kd, offset, (char *) load_ave, sizeof (load_ave))
 	  != sizeof (load_ave))
         {
           kvm_close (kd);
-          initialized = 0;
+          getloadavg_initialized = 0;
 	}
 #endif /* SUNOS_5 */
     }
 
-  if (offset == 0 || !initialized)
+  if (offset == 0 || !getloadavg_initialized)
     return -1;
 #endif /* LOAD_AVE_TYPE and not VMS */
 
@@ -606,6 +632,8 @@ getloadavg (loadavg, nelem)
     loadavg[elem++] = LDAV_CVT (load_ave[1]);
   if (nelem > 2)
     loadavg[elem++] = LDAV_CVT (load_ave[2]);
+
+#define LDAV_DONE
 #endif /* !LDAV_DONE && LOAD_AVE_TYPE */
 
 #ifdef LDAV_DONE
