@@ -22,19 +22,17 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "dep.h"
 #include "file.h"
 
-#if !defined (USG) && !defined (POSIX)
-#ifndef	sgi
-#include <sys/file.h>
-#endif
-#else
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
+#else
+#include <sys/file.h>
 #endif
 
 extern int try_implicit_rule ();
 
 
-/* Incremented when a file has been remade.  */
-unsigned int files_remade = 0;
+/* Incremented when a command is started (under -n, when one would be).  */
+unsigned int commands_started = 0;
 
 static int update_file (), update_file_1 (), check_dep (), touch_file ();
 static void remake_file ();
@@ -65,6 +63,17 @@ update_goal_chain (goals, makefiles)
 
   goals = copy_dep_chain (goals);
 
+  {
+    /* Clear the `changed' flag of each goal in the chain.
+       We will use the flag below to notice when any commands
+       have actually been run for a target.  When no commands
+       have been run, we give an "up to date" diagnostic.  */
+
+    struct dep *g;
+    for (g = goals; g != 0; g = g->next)
+      g->changed = 0;
+  }
+
   if (makefiles)
     /* Only run one job at a time.  */
     job_slots = 1;
@@ -87,6 +96,7 @@ update_goal_chain (goals, makefiles)
       g = goals;
       while (g != 0)
 	{
+	  unsigned int ocommands_started;
 	  int x;
 	  time_t mtime = MTIME (g->file);
 	  check_renamed (g->file);
@@ -103,14 +113,26 @@ update_goal_chain (goals, makefiles)
 		touch_flag = question_flag = just_print_flag = 0;
 	    }
 
+	  /* Save the old value of `commands_started' so we can compare later.
+	     It will be incremented when any commands are actually run.  */
+	  ocommands_started = commands_started;
+
 	  x = update_file (g->file, makefiles ? 1 : 0);
 	  check_renamed (g->file);
+
+	  /* Set the goal's `changed' flag if any commands were started
+	     by calling update_file above.  We check this flag below to
+	     decide when to give an "up to date" diagnostic.  */
+	  g->changed += commands_started - ocommands_started;
+
 	  if (x != 0 || g->file->updated)
 	    {
 	      int stop = 0;
+
 	      /* If STATUS was not already 1, set it to 1 if
 		 updating failed, or to 0 if updating succeeded.
 		 Leave STATUS as it is if no updating was done.  */
+
 	      if (status < 1)
 		{
 		  if (g->file->update_status != 0)
@@ -135,9 +157,27 @@ update_goal_chain (goals, makefiles)
 		    }
 		}
 
-	      g->file = g->file->prev;
-	      if (stop || g->file == 0)
+	      if (stop || g->file->prev == 0)
 		{
+		  /* If we have found nothing whatever to do for the goal,
+		     print a message saying nothing needs doing.  */
+
+		  if (!makefiles
+		      /* If the update_status is zero, we updated successfully
+			 or not at all.  G->changed will have been set above if
+			 any commands were actually started for this goal.  */
+		      && g->file->update_status == 0 && !g->changed
+		      /* Never give a message under -s or -q.  */
+		      && !silent_flag && !question_flag)
+		    {
+		      if (g->file->phony || g->file->cmds == 0)
+			message ("Nothing to be done for `%s'.",
+				 g->file->name);
+		      else
+			message ("`%s' is up to date.", g->file->name);
+		      fflush (stdout);
+		    }
+
 		  /* This goal is finished.  Remove it from the chain.  */
 		  if (lastgoal == 0)
 		    goals = g->next;
@@ -149,6 +189,15 @@ update_goal_chain (goals, makefiles)
 
 		  g = lastgoal == 0 ? goals : lastgoal->next;
 		}
+	      else if (g->file->updated)
+		/* This instance of the target is done being updated.
+		   Go to the next instance (:: rule).
+		   update_file cycles through all instances, but under -j,
+		   update_file can return while the file is running,
+		   then reap_children can change its command state and
+		   updated flag, leaving G->file done, but some of its
+		   other instances needing work.  */
+		g->file = g->file->prev;
 
 	      if (stop)
 		break;
@@ -191,7 +240,6 @@ update_file (file, depth)
 {
   register int status = 0;
   register struct file *f;
-  unsigned int ofiles_remade = files_remade;
 
   for (f = file; f != 0; f = f->prev)
     {
@@ -221,26 +269,6 @@ update_file (file, depth)
 	}
     }
 
-  /* For a top level target, if we have found nothing whatever to do for it,
-     print a message saying nothing needs doing.  */
-
-  /* Give a message iff updated successfully, and never under -s or -q.  */
-  if (status == 0 && !silent_flag && !question_flag
-      /* files_remade will have been incremented iff we actually
-	 ran any commands (under -n, if we would have).  */
-      && files_remade == ofiles_remade
-      /* Only give the diagnostic for top-level targets.
-	 The makefile chain run is done with DEPTH==1 precisely
-	 to avoid this message.  */
-      && depth == 0)
-    {
-      if (file->phony || file->cmds == 0)
-	message ("Nothing to be done for `%s'.", file->name);
-      else
-	message ("`%s' is up to date.", file->name);
-      fflush (stdout);
-    }
-
   return status;
 }
 
@@ -570,7 +598,7 @@ notice_finished_file (file)
 	     we don't want to do the touching.  */
 	  unsigned int i;
 	  for (i = 0; i < file->cmds->ncommand_lines; ++i)
-	    if (!file->cmds->lines_recurse[i])
+	    if (!(file->cmds->lines_flags[i] & COMMANDS_RECURSE))
 	      goto have_nonrecursing;
 	}
       else
@@ -609,7 +637,7 @@ notice_finished_file (file)
 	     We do this instead of just invalidating the cached time
 	     so that a vpath_search can happen.  Otherwise, it would
 	     never be done because the target is already updated.  */
-	  (void) f_mtime (d->file);
+	  (void) f_mtime (d->file, 0);
       }
 }
 
@@ -810,7 +838,7 @@ f_mtime (file, search)
      register struct file *file;
      int search;
 {
-  register time_t mtime;
+  time_t mtime;
 
   /* File's mtime is not known; must get it from the system.  */
 
@@ -882,11 +910,15 @@ f_mtime (file, search)
 	{
 	  /* If name_mtime failed, search VPATH.  */
 	  char *name = file->name;
-	  if (vpath_search (&name)
+	  if (vpath_search (&name, &mtime)
 	      /* Last resort, is it a library (-lxxx)?  */
 	      || (name[0] == '-' && name[1] == 'l'
-		  && library_search (&name)))
+		  && library_search (&name, &mtime)))
 	    {
+	      if (mtime != 0)
+		/* vpath_search and library_search store zero in MTIME
+		   if they didn't need to do a stat call for their work.  */
+		file->last_mtime = mtime;
 	      rename_file (file, name);
 	      check_renamed (file);
 	      return file_mtime (file);
@@ -925,8 +957,9 @@ name_mtime (name)
    directories.  */
 
 static int
-library_search (lib)
+library_search (lib, mtime_ptr)
      char **lib;
+     time_t *mtime_ptr;
 {
   static char *dirs[] =
     {
@@ -937,6 +970,7 @@ library_search (lib)
     };
 
   char *libname = &(*lib)[2];	/* Name without the `-l'.  */
+  time_t mtime;
 
   /* Buffer to construct possible names in.  */
   char *buf = xmalloc (sizeof (LIBDIR) + 8 + strlen (libname) + 4 + 2 + 1);
@@ -945,16 +979,19 @@ library_search (lib)
   /* Look first for `libNAME.a' in the current directory.  */
 
   sprintf (buf, "lib%s.a", libname);
-  if (name_mtime (buf) != (time_t) -1)
+  mtime = name_mtime (buf);
+  if (mtime != (time_t) -1)
     {
       *lib = buf;
+      if (mtime_ptr != 0)
+	*mtime_ptr = mtime;
       return 1;
     }
 
   /* Now try VPATH search on that.  */
 
   file = buf;
-  if (vpath_search (&file))
+  if (vpath_search (&file, mtime_ptr))
     {
       free (buf);
       *lib = file;
@@ -966,9 +1003,12 @@ library_search (lib)
   for (dp = dirs; *dp != 0; ++dp)
     {
       sprintf (buf, "%s/lib%s.a", *dp, libname);
-      if (name_mtime (buf) != (time_t) -1)
+      mtime = name_mtime (buf);
+      if (mtime != (time_t) -1)
 	{
 	  *lib = buf;
+	  if (mtime_ptr != 0)
+	    *mtime_ptr = mtime;
 	  return 1;
 	}
     }
